int Remove_From_Heap (
  Priority_Heap *Heap,
  int *Value)
{
  /*
   *  Removes the top item from the heap
   *  then the data in the last node is moved to the top
   *  of the heap. Finally the heap is adjusted down to
   *  maintain a valid balanced heap.
   *  The heap is dynamic with the largest value at the
   *  top of the heap
   *
   *  The heap is maintained as a balanced tree.
   *  A circular deque holding pointers to the nodes in the heap
   *  which have NULL left or right pointers is also maintained. 
   *  This next connection list has
   *  a pointer to the tail (right most end) of the list.
   *
   *  This function returns ERROR_DETECTED if any errors are detected
   *  and returns NO_ERRORS_DETECTED if no errors are detected
   */
  
  int Function_Flag = NO_ERRORS_DETECTED;
  int Node_Value = 0;
  Tree_Node_Pointer Ptr_Extracted_Node = NULL;
  
  int Remove_From_Tail_Of_List (
    List_Node_Pointer *Ptr_Tail,
    Tree_Node_Pointer *Ptr_Tree_Node);
 
  int Insert_At_Head_Of_List (
    List_Node_Pointer *Ptr_Tail,
    Tree_Node_Pointer Ptr_Tree_Node);

  void Adjust_Heap_Down (Tree_Node_Pointer Ptr_P);
  
  /*  
   *  Remove the first available List_Node_Pointer from the "Heap".
   *  The pointer Ptr_Extracted_Node points to the next available node to 
   *  be extracted (from the Tail). 
   *  Make sure the extracted node from the List_Node_Pointer is a proper
   *  child.  Otherwise, flag ERROR_DETECTED.
   *  Return data from extracted tree node head to the content of the
   *  local copy integer Node_Value.
   */

  Function_Flag = Remove_From_Tail_Of_List (
                    &(Heap->Ptr_Tail_Of_Next_Connection_Deque),
                    &Ptr_Extracted_Node);

  if (Function_Flag != NO_ERRORS_DETECTED)
  {
    return ERROR_DETECTED;
  } 
  
  /*  Checks for proper child  */

  if (   (Ptr_Extracted_Node->Ptr_Left != NULL)
      && (Ptr_Extracted_Node->Ptr_Right != NULL))
  {
    return ERROR_DETECTED;
  }
  
  /*  Extract value from head node of tree structure  */

  Node_Value = Heap->Ptr_Top_Of_Heap->Data;

  /*  
   *  Special case:  Last node around; the head node.
   *  On the occurance of the last node, free the corresponding 
   *  node and return the value Node_Value.
   *  Don't forget to set "Heap" head to NULL, "Tail" to NULL!
   */

  if (Ptr_Extracted_Node->Ptr_Up == NULL)
  {
    Heap->Ptr_Top_Of_Heap = NULL;
    Heap->Ptr_Tail_Of_Next_Connection_Deque = NULL;
    free(Ptr_Extracted_Node);
    Ptr_Extracted_Node = NULL;
    *Value = Node_Value;
    return NO_ERRORS_DETECTED;  
  }

  /* 
   *  Make sure parent node doesn't point to extracted node 
   *  any longer.  If a right node case occur, then insert a new list
   *  node into the highest priority heap.
   *  Finally, exchange extracted node data with head and remove any
   *  traces of the extracted node.
   */

  if (Ptr_Extracted_Node->Ptr_Up->Ptr_Right == Ptr_Extracted_Node)
  {
    /*  
     *  Right node case.  Insert parent in top priority list in the
     *  heap.  Set pointer of parent's right to NULL.
     */

    Ptr_Extracted_Node->Ptr_Up->Ptr_Right = NULL;
    Function_Flag = Insert_At_Head_Of_List (
                      &Heap->Ptr_Tail_Of_Next_Connection_Deque,
                      Ptr_Extracted_Node->Ptr_Up);
  }
  else if (Ptr_Extracted_Node->Ptr_Up->Ptr_Left == Ptr_Extracted_Node)
  {
    /*
     *  Left node case.  Unconnect parent's left node pointer 
     *  and point it to NULL.
     */
   
    Ptr_Extracted_Node->Ptr_Up->Ptr_Left = NULL;
  }
  else
  {
     return ERROR_DETECTED;
  }

  /* Exchange data */

  Heap->Ptr_Top_Of_Heap->Data = Ptr_Extracted_Node->Data;
  Ptr_Extracted_Node->Ptr_Up = NULL;
  free (Ptr_Extracted_Node);
  Ptr_Extracted_Node = NULL;

  Adjust_Heap_Down (Heap->Ptr_Top_Of_Heap);

  /*  
   *  Assign pointer Value with value of local variable
   *  Node_Value.
   */

  *Value = Node_Value;
  return NO_ERRORS_DETECTED;

} /* Remove_From_Heap */

